#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

install_requires = \
['jsonschema>=3,<4']

extras_require = \
{'test': ['pytest>=6,<7', 'pytest-cov>=2,<3']}

setup(name='cascade-config',
      version='0.3.0',
      description='Cascading configuration from the CLI and config files.',
      author='RalfG',
      author_email='ralf@gabriels.dev',
      url='https://github.com/RalfG/cascade-config',
      py_modules=['cascade_config'],
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
